import 'dart:io';

class CLI {
  static Future<ProcessResult> run(
    String cmd,
    List<String> args, {
    bool throwOnError = true,
    String? workingDirectory,
  }) async {
    final result = await Process.run(
      cmd,
      args,
      workingDirectory: workingDirectory,
      runInShell: true,
    );

    print(result.stderr);
    print(result.stdout);

    if (throwOnError) {
      if (result.stderr != null) {
        _throwIfProcessFailed(result, cmd, args);
      }
    }
    return result;
  }

  static void _throwIfProcessFailed(
    ProcessResult pr,
    String process,
    List<String> args,
  ) {
    if (pr.exitCode != 0) {
      final values = {
        'Standard out': pr.stdout.toString().trim(),
        'Standard error': pr.stderr.toString().trim()
      }..removeWhere((k, v) => v.isEmpty);

      var message = 'Unknown error';
      if (values.isNotEmpty) {
        message = values.entries.map((e) => '${e.key}\n${e.value}').join('\n');
      }

      throw ProcessException(process, args, message, pr.exitCode);
    }
  }

  static Future<bool> checkIfFlutterAvailable() async {
    try {
      await run(
        'flutter',
        ['--version'],
      );
      return true;
    } catch (e) {
      return false;
    }
  }

  static Future<bool> createNewProject(String projectPath, String projectName) async {
    try {
      await run(
        'flutter',
        ["create", projectPath, "--project-name", projectName],
      );
      return true;
    } catch (e) {
      return false;
    }
  }

  static Future<bool> getPackages() async {
    try {
      await run(
        'flutter',
        ["pub", "get"],
      );
      return true;
    } catch (e) {
      return false;
    }
  }

  static Future<bool> runDartGenerator(String directory) async {
    try {
      await run(
        'flutter',
        ["pub", "run", "build_runner", "build"],
        workingDirectory: directory,
      );
      return true;
    } catch (e) {
      return false;
    }
  }

  static Future<bool> runFormatter(String directory) async {
    try {
      await run(
        'flutter',
        ["format", "."],
        workingDirectory: directory,
      );
      return true;
    } catch (e) {
      return false;
    }
  }
}
