import 'dart:io';

import 'package:cli_a2soft/bricks/project_code_samples_bundle.dart';
import 'package:cli_a2soft/errors/cli_errors.dart';
import 'package:mason/mason.dart';
import 'package:path/path.dart' as p;
import 'package:args/command_runner.dart';
import 'package:cli_a2soft/cli/cli.dart';
import 'package:cli_a2soft/cli/cli_mixin.dart';

part 'install_packages.dart';
part 'install_bricks.dart';

class CreateCommand extends Command with CLIMixin {
  CreateCommand();

  @override
  String get description => "Создает проект по шаблону A2Soft";

  @override
  String get name => "create";

  static const _ignoredDirectories = {
    'ios',
    'android',
    'windows',
    'linux',
    'macos',
    '.symlinks',
    '.plugin_symlinks',
    '.dart_tool',
    'build',
  };

  bool _isPubspec(FileSystemEntity entity) {
    final segments = p.split(entity.path).toSet();
    if (segments.intersection(_ignoredDirectories).isNotEmpty) return false;
    if (entity is! File) return false;
    return p.basename(entity.path) == 'pubspec.yaml';
  }

  @override
  Future<void> run() async {
    /// Проверяем, установлен ли на машине Flutter и можем ли мы до него добраться
    final isFlutterAvailable = await CLI.checkIfFlutterAvailable();
    if (!isFlutterAvailable) {
      throw FlutterNotInstalledException();
    } else {
      printSuccess("Flutter SDK найден в системе\n");
    }

    /// Получаем пользовательский инпут
    final projectPath = getUserInput<String>(
      helpPhrase: "Путь для проекта",
      defaultValue: './a2soft_project',
    );
    final projectName = getUserInput<String>(
      helpPhrase: "Название для проекта",
      defaultValue: 'a2soft_project',
    );
    final database = getUserInput<bool>(
      helpPhrase: "Подключить к проекту базу данных?",
      defaultValue: false,
    );
    final localization = getUserInput<bool>(
      helpPhrase: "Подключить к проекту локализацию?",
      defaultValue: false,
    );
    final cookieManager = getUserInput<bool>(
      helpPhrase: "Подключить к проекту менеджер куков?",
      defaultValue: false,
    );
    final loaderKit = getUserInput<bool>(
      helpPhrase: "Подключить к проекту библиотеку лоадеров?",
      defaultValue: false,
    );
    final globalLoader = getUserInput<bool>(
      helpPhrase: "Подключить к проекту глобальный лоадер?",
      defaultValue: false,
    );
    final nativeDialog = getUserInput<bool>(
      helpPhrase: "Подключить к проекту нативные диалоги?",
      defaultValue: false,
    );
    final cacheInterceptor = getUserInput<bool>(
      helpPhrase: "Подключить к проекту кеширующий обработчик?",
      defaultValue: false,
    );

    /// Создаем проект командой [flutter create <DESTINATION>]
    final isCreated = await CLI.createNewProject(projectPath, projectName);
    if (isCreated) {
      printSuccess("Проект успешно создан по пути: $projectPath \n");
    } else {
      throw CreateProjectException();
    }

    /// Находим pubspec в проекте
    final pubspec = File("$projectPath/pubspec.yaml");
    if (!(pubspec.existsSync() && _isPubspec(pubspec))) {
      throw PubspecNotFoundException();
    } else {
      printSuccess("pubspec.yaml найден в созданном проекте\n");
    }

    /// Устанавливаем  пакеты
    installBasePackages(pubspec);
    if (database) {
      installDatabase(pubspec);
    }
    if (cacheInterceptor) {
      installDioCacheInterceptor(pubspec);
    }
    if (localization) {
      installLocalization(pubspec);
    }
    if (nativeDialog) {
      installAdaptiveDialog(pubspec);
    }
    if (globalLoader) {
      installGlobalLoader(pubspec);
    }
    if (loaderKit) {
      installLoaderKit(pubspec);
    }
    if (cookieManager) {
      installCookieManager(pubspec);
    }

    // print("Генерируем BLoC с примером...");
    // await generateBLoCSamples(Directory(projectPath), "Car");
    // printSuccess("Готово!");

    print("Вставляем code samples...");
    await generateProjectSamples(
      projectDir: Directory(projectPath),
      projectName: projectName,
      enableLocalization: localization,
      enableDatabase: database,
    );
    printSuccess("Готово!");

    print("Получение пакетов...");
    await CLI.getPackages();
    printSuccess("Готово!");

    await Future.delayed(Duration(seconds: 2));

    print("Генерация файлов...");
    await CLI.runDartGenerator(projectPath);
    await CLI.runFormatter(projectPath);
    printSuccess("Готово!");
  }
}
